void convertRecTimeTableFromXmlToSkip(string timeData, Skip &recTimeTable)
{
	if null timeData then return

	Buffer hBuffer = create
	hBuffer += timeData
	int startPos = 0
	string aRecord = null
	Buffer aBuffer = create
	while (getValueBetweenTags(startPos, hBuffer, H_X_REC_TIME, aRecord))  // loop for each recording time
	{
		setempty aBuffer
		aBuffer = aRecord
		string sectionId = ""
		string recTime = ""
		int aPos = 0
		getValueBetweenTags(aPos, aBuffer, H_X_SECTION, sectionId)
		getValueBetweenTags(aPos, aBuffer, H_X_TIME, recTime)
		csintDebug_("Loading Time string is " recTime)
		Date loadingTime
		if null recTime then
		{
			csintDebug_("Error: recording time is null.")
			loadingTime = recTime

		}
		else if isValidInt(recTime) then
		{
			int dateNumber = intOf(recTime)
			loadingTime = dateOf(dateNumber)
		}
		else
		{
			loadingTime = recTime
		}
		csintDebug_("Loading Date is " loadingTime "")
		put(recTimeTable, sectionId, loadingTime)
	}
	delete hBuffer
	delete aBuffer
}

// ========================================================================
void convertObjMappingsFromXmlToSkip(string mappingData, Skip &newObjMappings)
{
	if null mappingData then return

	Buffer hBuffer = create
	hBuffer += mappingData
	int startPos = 0
	string aRecord = null
	Buffer aBuffer = create
	while (getValueBetweenTags(startPos, hBuffer, H_X_OBJ, aRecord))  // loop for each obj
	{
		setempty aBuffer
		aBuffer = aRecord
		string currentId = ""
		string firstId = ""
		int aPos = 0
		getValueBetweenTags(aPos, aBuffer, H_X_CURRENT_ID, currentId)
		getValueBetweenTags(aPos, aBuffer, H_X_FIRST_ID, firstId)
		put(newObjMappings, currentId, firstId)
	}
	delete hBuffer
	delete aBuffer
}

// ========================================================================
void convertObjMappingsToXml(Buffer &xmlBuffer, Skip newObjMappings)
{
	xmlBuffer += XMLstartTag(H_X_NEW_OBJ_MAPPING)
	xmlBuffer += "\n"

	string oldObjId = null
	string newObjId = null
	for oldObjId in newObjMappings do
	{
		xmlBuffer += XMLstartTag(H_X_OBJ)

		newObjId = (string key newObjMappings)
		insertObjectDataToXml(xmlBuffer, H_X_CURRENT_ID, newObjId)
		insertObjectDataToXml(xmlBuffer, H_X_FIRST_ID,   oldObjId)

		xmlBuffer += XMLendTag(H_X_OBJ)
		xmlBuffer += "\n"
	}

	xmlBuffer += XMLendTag(H_X_NEW_OBJ_MAPPING)
	xmlBuffer += "\n"
}

// ========================================================================
void save_loading_data(Module m, string rcpID, Skip newObjMappings, Date loadEndTime, Skip &recTimeTable)
{
	if !null loadEndTime then
	{
		csintDebug_("save_loading_data(): There is UML merge happened before, the loading time is set ahead as " loadEndTime "")
		if find(recTimeTable, "0") then delete(recTimeTable, "0")
		put(recTimeTable, "0", loadEndTime)
	}

	Buffer xmlBuffer = create
	xmlBuffer += XMLstartTag(H_X_LOAD)
	xmlBuffer += "\n"

	xmlBuffer += XMLstartTag(H_X_RCP_IN_USE)
	xmlBuffer += escapeXML(rcpID)
	xmlBuffer += XMLendTag(H_X_RCP_IN_USE)
	xmlBuffer += "\n"
/*
	xmlBuffer += XMLstartTag(H_X_MRC_NEW_CVID)
	xmlBuffer += ""	// this will be mrc new cvid
	xmlBuffer += XMLendTag(H_X_MRC_NEW_CVID)
	xmlBuffer += "\n"

	xmlBuffer += XMLstartTag(H_X_MRC_ORIG_CVID)
	xmlBuffer += ""	// this will be mrc orig cvid
	xmlBuffer += XMLendTag(H_X_MRC_ORIG_CVID)
	xmlBuffer += "\n"
*/
	xmlBuffer += XMLstartTag(H_X_REC_TIMES)
	xmlBuffer += "\n"
	Date recDate = null
	for recDate in recTimeTable do
	{
		string sAbsNo = (string key recTimeTable)
		int dateNumber = intOf(recDate)

		xmlBuffer += XMLstartTag(H_X_REC_TIME)

		xmlBuffer += XMLstartTag(H_X_SECTION)
		xmlBuffer += sAbsNo
		xmlBuffer += XMLendTag(H_X_SECTION)

		xmlBuffer += XMLstartTag(H_X_TIME)
		xmlBuffer += dateNumber ""
		xmlBuffer += XMLendTag(H_X_TIME)

		xmlBuffer += XMLendTag(H_X_REC_TIME)

		xmlBuffer += "\n"
	}
	xmlBuffer += XMLendTag(H_X_REC_TIMES)
	xmlBuffer += "\n"

	convertObjMappingsToXml(xmlBuffer, newObjMappings)

	xmlBuffer += XMLendTag(H_X_LOAD)

	// save to user config
	string mid = uniqueID(item(fullName(m)))
	string parentDir = LOADING_CACHE_DIR
	if !fileExists(parentDir) then mkdir(parentDir)
	parentDir = LOADING_CACHE_DIR "\\csint_" mid
	if !fileExists(parentDir) then mkdir(parentDir)

	string hFileName = LOADING_CACHE_DIR "\\csint_" mid "\\" LOADING_DATA_FILE
	if fileExists(hFileName) then
	{
		deleteFile(hFileName)
	}
	Stream out = write(hFileName, CP_UTF8)
	out << xmlBuffer
	close(out)
	
	delete xmlBuffer
}

// ========================================================================
void read_loading_data(Module m, string &rcp_in_use, Skip &recTimeTable, Skip &objMappings)
{
	csintDebug_("read_loading_data():")
	Buffer data_buffer = create

	string mid = uniqueID(item(fullName(m)))
	string hFileName = LOADING_CACHE_DIR "\\csint_" mid "\\" LOADING_DATA_FILE
	if fileExists(hFileName) then
	{
		readDoorsFile(hFileName, data_buffer)

		csintDebug_(stringOf data_buffer)
		int startPos = 0
		string hText = null
		if (getValueBetweenTags(startPos, data_buffer, H_X_LOAD, hText)) then
		{
			// now hText contains the obj records without H_X_LOAD tag
			Buffer b2 = create
			int bStart = 0
			b2 = hText
			string result = ""
			if (getValueBetweenTags(bStart, b2, H_X_RCP_IN_USE, result)) then
			{
				csintDebug_(result)
				rcp_in_use = result
			}
/*
			result = ""
			if (getValueBetweenTags(bStart, b2, H_X_MRC_NEW_CVID, result)) then
			{
				csintDebug_("MRC New cvid:")
				csintDebug_(result)
				mrc_new_cvid = result
			}
			result = ""
			if (getValueBetweenTags(bStart, b2, H_X_MRC_ORIG_CVID, result)) then
			{
				csintDebug_("MRC Orig cvid:")
				csintDebug_(result)
				mrc_orig_cvid = result
			}
*/
			result = ""
			if (getValueBetweenTags(bStart, b2, H_X_REC_TIMES, result)) then
			{
				csintDebug_("recording time table string:")
				csintDebug_(result)
				convertRecTimeTableFromXmlToSkip(result, recTimeTable)
			}
			result = ""
			if (getValueBetweenTags(bStart, b2, H_X_NEW_OBJ_MAPPING, result)) then
			{
				csintDebug_("obj mappings string:")
				csintDebug_(result)
				convertObjMappingsFromXmlToSkip(result, objMappings)
			}
			delete b2
		}
	}
	else
	{
		// error
		csintDebug_ ("Error: No loading data found!!")
	}

	delete data_buffer
}
