// -----------------------------------------------------------------------
// String substitution utility
//
// We use '%' and one char after it as the indicator string for substitution
// with the following rules:
//
//	indicator string		replaced with		comment
// -----------------------------------------------------------------------
//	%%					%				This is the case if you want to include real '%' char.
//	%[1-9]				arg[1-9]			For argument 1 ~ 9 
//	%[^%1-9]				null string		For the case of '%' is followed by any char other than '%' or '1' through '9'.
//	%0					null string		Just want to remind you that there is NO argument Zero.
//	%[end of base string]	null string		In case you are not clear what will happen when the '%' is placed in the end of base string.
//
// The appearance of the "%[1-9]" don't need to be in order.
//
// You can have more than one the same indicator strings in your base string.
//		In other words, you can have more than one "%1" in your base string.  They will all be replaced with arg1.
//		The same for "%[2-9]", they will all be replaced with their corresponding arg[2-9].
//		The same for "%%", you can have any number of "%%" in your base sting, they will all be replaced with single "%".
//
// The above substitution only apply to the base string.
//		The substitute strings ( or the argument strings ) will not be processed.
//		In other words, the argument strings can contain char '%' and will not be substituted.
//
// for example:
//
// getCsintString("%0---%a@@@%%++", "A", "B")					will return	"---@@@%++"
// getCsintString("%", "Hello", "World")						will return	""
// getCsintString("%3 %1 %2.", "took a walk with", "my dog", "I")	will return	"I took a walk with my dog."
// getCsintString("My %1 is still my %1.", "money")				will return	"My money is still my money."
// getCsintString("----%1----", "%===%")						will return	"----%===%----"
// -----------------------------------------------------------------------


Regexp reCsintPercent = regexp "%"			// the string substitution char
Regexp reCsintAfterEquals = regexp "=.*"    // From equal sign to the end of the string
Regexp reCsintNewline	= regexp "\\\\n"    // \n
string csintStringTableFilename = (getenv "DOORSHOME") "\\lib\\dxl\\addins\\synergy\\misc\\csintStringTable.txt"
Skip csintStringTable = createString

// -----------------------------------------------------------------------
// This function takes the following 4 args:
//	dataBuffer	- in/out	: hold the data string need to be processed.
//	pos			- in		: point to '%' found
//	replaceWith	- in		: the substitution string
//	workingBuffer	- in/out	: A reuseable temp buffer
//
// return a int value to indicate where to start search for next iteration
// -----------------------------------------------------------------------
int csintSubstituteOneString(Buffer &dataBuffer, int pos, string replaceWith, Buffer &workingBuffer)
{
	setempty workingBuffer
	combine(workingBuffer, dataBuffer, 0, pos-1)
	workingBuffer += replaceWith
	int ptr = length workingBuffer	// update the ptr here
	combine(workingBuffer, dataBuffer, pos+2)
	setempty dataBuffer
	combine(dataBuffer, workingBuffer, 0)
	return ptr
}

void repairNewlines(Buffer &dataBuffer)
{
	
	int ptr = 0
	int pos = 0
	Buffer workingBuffer = create
	while (true)
	{
		if (search(reCsintNewline, dataBuffer, ptr))
		{
			pos = ptr + start(0)
			setempty workingBuffer
			combine(workingBuffer, dataBuffer, 0, pos-1)
			workingBuffer += '\n'
			ptr = length workingBuffer
			combine(workingBuffer, dataBuffer, pos+2)
			setempty dataBuffer
			dataBuffer = workingBuffer
		}
		else
		{
			break
		}
	}
	delete workingBuffer
}

int findEqualSign(Buffer tmpBufferStr)
{
	int tmp_length = length(tmpBufferStr)
	int i
	char ch
	for (i=0; i<tmp_length; i++)
	{
		ch = tmpBufferStr[i]
		if (ch == '=') then
		{
			return i
		}
	}
	return -1
}

// This function will read strings from a string table file rather than declaring them globally
// Strings are cached in a global skip and will be fetched from the skip first if possible
string getCsintStringFromStringTable(string baseString)
{	
	string retval = ""
	if (find(csintStringTable, baseString, retval))
	{
		return retval
	}
	else
	{
		Buffer read_buffer = create
		Buffer tmpBuffer = create
		Buffer whitespace = create

		Stream f
		f = read(csintStringTableFilename, CP_UTF8)
		int ptr = 0
		while (!end f)
		{
			f -> read_buffer
			if (contains(read_buffer, baseString, 0) >= 0)
			{ //
				setempty tmpBuffer
				tmpBuffer = read_buffer				
				int pos = findEqualSign(tmpBuffer)
				if (pos > -1)
				//if (search(reCsintAfterEquals, tmpBuffer, ptr))
				{
					pos = pos + 1
					//int pos = ptr + start(0) + 1
					setempty whitespace
					whitespace = tmpBuffer[pos:]
					pos = firstNonSpace(whitespace)
					setempty tmpBuffer
					tmpBuffer = whitespace[pos:]
				//	repairNewlines(tmpBuffer)
					retval = stringOf(tmpBuffer)
					put(csintStringTable, baseString, retval)
					break
				}
			}
			setempty read_buffer
		}
		close f
		delete read_buffer
		delete tmpBuffer
		delete whitespace
	}
	if (retval == "")
	{
		ack "Didn't find string " baseString " in string table"
	}
	return retval
}

// -----------------------------------------------------------------------
// The following over-loaded functions are basicly the same. The only difference
// is the number of the substitution args.... from 9 down to 1
// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
{
	string lookedupValue = getCsintStringFromStringTable(baseString)
	Buffer dataBuffer = create
	Buffer workingBuffer = create
	dataBuffer += lookedupValue
	int ptr = 0
	while (true)
	{
		int dataLength = length(dataBuffer)
		if dataLength == 0 then break

		// search from ptr
		if search(reCsintPercent, dataBuffer, ptr) then
		{
			int pos = ptr + start(0)
			if pos == (dataLength-1) then
			{
				// some one put the % in the end of base string... don't do it!! OK~~~  ;(

				setempty workingBuffer
				combine(workingBuffer, dataBuffer, 0, pos-1)	// get rid of the tailing %
				setempty dataBuffer
				combine(dataBuffer, workingBuffer, 0)
				break
			}
			string followChar = dataBuffer[pos+1:pos+1]
			if followChar == "%" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, "%", workingBuffer)	// update the ptr here
			}
			else if followChar == "1" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg1, workingBuffer)	// update the ptr here
			}
			else if followChar == "2" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg2, workingBuffer)	// update the ptr here
			}
			else if followChar == "3" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg3, workingBuffer)	// update the ptr here
			}
			else if followChar == "4" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg4, workingBuffer)	// update the ptr here
			}
			else if followChar == "5" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg5, workingBuffer)	// update the ptr here
			}
			else if followChar == "6" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg6, workingBuffer)	// update the ptr here
			}
			else if followChar == "7" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg7, workingBuffer)	// update the ptr here
			}
			else if followChar == "8" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg8, workingBuffer)	// update the ptr here
			}
			else if followChar == "9" then
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, arg9, workingBuffer)	// update the ptr here
			}
			else
			{
				ptr = csintSubstituteOneString(dataBuffer, pos, "", workingBuffer)	// update the ptr here
			}
		}
		else
		{
			break
		}
	}
	delete workingBuffer
	string resultString = stringOf dataBuffer
	delete dataBuffer
	return resultString
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)
{
	return getCsintString(baseString, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4, arg5, arg6, arg7)
{
	return getCsintString(baseString, arg1, arg2, arg3, arg4, arg5, arg6, arg7, "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4, arg5, arg6)
{
	return getCsintString(baseString, arg1, arg2, arg3, arg4, arg5, arg6, "", "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4, arg5)
{
	return getCsintString(baseString, arg1, arg2, arg3, arg4, arg5, "", "", "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3, arg4)
{
	return getCsintString(baseString, arg1, arg2, arg3, arg4, "", "", "", "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2, arg3)
{
	return getCsintString(baseString, arg1, arg2, arg3, "", "", "", "", "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1, arg2)
{
	return getCsintString(baseString, arg1, arg2, "", "", "", "", "", "", "")
}

// -----------------------------------------------------------------------
string getCsintString(string baseString, string arg1)
{
	return getCsintString(baseString, arg1, "", "", "", "", "", "", "", "")
}

string getCsintString(string baseString)
{
	return getCsintStringFromStringTable(baseString)
}
