//==============================================================================
//
//==============================================================================
void baseSearchAndReplace(Regexp &re, string replaceWith, Buffer &s, Buffer new)
{
	int pos = 0
	int ptr = 0
	while (search(re, s, ptr))
	{
		pos = ptr + start(0)
		combine(new, s, ptr, pos - 1) 
		new += replaceWith 
		ptr = ptr + end(0) + 1   
	}
	combine(new,s,ptr)
	setempty(s)
	combine(s,new,0)
}

void searchAndReplace(Regexp &re, string replaceWith, Buffer &s, Buffer new)
{
	if (search(re, s, 0)) then
	{
		setempty new
		baseSearchAndReplace(re, replaceWith, s, new)
	}  
}

void searchAndReplace(Regexp &re, string replaceWith, Buffer &s)
{
	if (search(re, s, 0)) then
	{
		Buffer new = create 
		baseSearchAndReplace(re, replaceWith, s, new)
		delete(new)
	}  
}

//==============================================================================
// Convert a string to xml format like
//==============================================================================
void escapeXML(Buffer &b)
{
	searchAndReplace(reAmp, "&amp;", b)
	searchAndReplace(reLt, "&lt;", b)
	searchAndReplace(reGt, "&gt;", b)
	searchAndReplace(reQuot, "&quot;", b)
	searchAndReplace(reApos, "&apos;", b)
}

//==============================================================================
//
//==============================================================================
string escapeXML(string s)
{
	if null s then return ""
	if s == "" then return ""

	Buffer b = create
	b += s
	escapeXML(b)	
	string retval = stringOf(b)
	delete b
	return retval
}

//==============================================================================
// This version re-use buffer, output is in buffer
//==============================================================================
void bEscapeXML(string s, Buffer &b)
{
	setempty b
	if !null s then
	{
		b += s
		escapeXML(b)
	}
}

//==============================================================================
//
//==============================================================================
void unescapeXML(Buffer &xml)
{
     searchAndReplace(reuLt, "<", xml)
     searchAndReplace(reuGt, ">", xml)
     searchAndReplace(reuAmp, "&", xml)
	 searchAndReplace(reuQuote, "\"", xml)
	 searchAndReplace(reuApos, "'", xml)
}

//==============================================================================
// This version re-use buffer, output is in buffer
//==============================================================================
void bUnescapeXML(string s, Buffer &xml)
{
	setempty xml
	if !null s then
	{
		xml += s
		unescapeXML(xml)
	}
}

//==============================================================================
//
//==============================================================================
string XMLstartTag(string tagName)
{
	Buffer b = create
	b += xTAG_OPEN
	b += tagName
	b += xTAG_CLOSE
	string result = stringOf(b)
	delete b
	return result
}

void bufferWriteXMLstartTag(Buffer &b, string tagName)
{
	b += xTAG_OPEN
	b += tagName
	b += xTAG_CLOSE
}

void confWriteXmlStartTag(ConfStream out, string tagName)
{
	out << xTAG_OPEN
	out << tagName
	out << xTAG_CLOSE
}

void fileWriteXmlStartTag(Stream out, string tagName)
{
	out << xTAG_OPEN
	out << tagName
	out << xTAG_CLOSE
}

//==============================================================================
//
//==============================================================================
string XMLendTag(string tagName)
{
	Buffer b = create
	b += xTAG_OSLASH
	b += tagName
	b += xTAG_CLOSE
	string result = stringOf(b)
	delete b
	return result
}

void bufferWriteXMLendTag(Buffer &b, string tagName)
{
	b += xTAG_OSLASH
	b += tagName
	b += xTAG_CLOSE
}

void confWriteXmlEndTag(ConfStream out, string tagName)
{
	out << xTAG_OSLASH
	out << tagName
	out << xTAG_CLOSE
}

void fileWriteXmlEndTag(Stream out, string tagName)
{
	out << xTAG_OSLASH
	out << tagName
	out << xTAG_CLOSE
}

// ========================================================================
void insertObjectDataToXml(Buffer &b, string tag, string data)
{
	bufferWriteXMLstartTag(b, tag)
	b += data
	bufferWriteXMLendTag(b, tag)
}

void confWriteObjectDataToXml(ConfStream out, string tag, string data)
{
	confWriteXmlStartTag(out, tag)
	out << data
	confWriteXmlEndTag(out, tag)
}

void fileWriteDataToXml(Stream out, string tag, string data)
{
	fileWriteXmlStartTag(out, tag)
	out << data
	fileWriteXmlEndTag(out, tag)
}

void fileWriteBufferToXml(Stream out, string tag, Buffer data)
{
	fileWriteXmlStartTag(out, tag)
	out << data
	fileWriteXmlEndTag(out, tag)
}

// ========================================================================
void fileWriteKeyValueToXml(Stream out, string keyName, string keyValue)
{
	fileWriteDataToXml(out, xTAG_KEY, keyName)
	fileWriteDataToXml(out, xTAG_VALUE, keyValue)
	out << "\n"
}

void fileWriteKeyEscapedValueToXml(Stream out, string keyName, string keyValue, Buffer tempBuffer)
{
	fileWriteDataToXml(out, xTAG_KEY, keyName)
	bEscapeXML(keyValue, tempBuffer)
	fileWriteBufferToXml(out, xTAG_VALUE, tempBuffer)
	out << "\n"
}

// ========================================================================
Skip convertPropertiesXml2Skip(Buffer &dataBuffer, Buffer &tempBuffer)
{
	Skip propertiesSkip = createString
	int iPos = 0
	string keyString
	string valueString
	while (getValueBetweenTags(iPos, dataBuffer, xTAG_KEY, keyString))
	{
		bUnescapeXML(keyString, tempBuffer)
		keyString = stringOf(tempBuffer)

		if (getValueBetweenTags(iPos, dataBuffer, H_X_VALUE, valueString)) then
		{
			bUnescapeXML(valueString, tempBuffer)
			valueString = stringOf(tempBuffer)

			put(propertiesSkip, keyString, valueString)
		}
	}
	return propertiesSkip
}

// ========================================================================
// we should always try to use this version, which reuses the buffer
string attributeNameToValidFileName(string attrName, Buffer b, new)
{
	setempty b
	b += attrName

	searchAndReplace(reAmp,		"&amp;",		b, new)
	searchAndReplace(reBackSlash, "&#92;",		b, new)
	searchAndReplace(reSlash,	"&#47;",		b, new)
	searchAndReplace(reColon,	"&#58;",		b, new)
	searchAndReplace(reStar,		"&#42;",		b, new)
	searchAndReplace(reQuestion,	"&#63;",		b, new)
	searchAndReplace(reQuot,		"&#quot;",	b, new)
	searchAndReplace(reLt,		"&lt;",		b, new)
	searchAndReplace(reGt,		"&gt;",		b, new)
	searchAndReplace(rePipe,		"&#124;",		b, new)

	return stringOf b
}

// if we can not reuse the buffer, use this version
string attributeNameToValidFileName(string attrName)
{
	Buffer b = create
	Buffer new = create
	string validFileName = attributeNameToValidFileName(attrName, b, new)
	delete b
	delete new
	return validFileName
}

// ========================================================================
// we should always try to use this version, which reuses the buffer
string validFileNameToAttributeName(string validFileName, Buffer b, new)
{
	setempty b
	b += validFileName

	searchAndReplace(reuBackSlash,	"\\",	b, new)
	searchAndReplace(reuSlash,		"/",		b, new)
	searchAndReplace(reuColon,		":",		b, new)
	searchAndReplace(reuStar,		"*",		b, new)
	searchAndReplace(reuQuestion,		"?",		b, new)
	searchAndReplace(reuQuot,		"\"",	b, new)
	searchAndReplace(reuLt,			"<",		b, new)
	searchAndReplace(reuGt,			">",		b, new)
	searchAndReplace(reuPipe,		"|",		b, new)
	searchAndReplace(reuAmp,			"&",		b, new)

	return stringOf b
}

// if we can not reuse the buffer, use this version
string validFileNameToAttributeName(string validFileName)
{
	Buffer b = create
	Buffer new = create
	string attrName = validFileNameToAttributeName(validFileName, b, new)
	delete b
	delete new
	return attrName
}

// ========================================================================
string escapeEscapeChar(string inputString, Buffer b, new)
{
	if null inputString then return ""

	b = inputString
	searchAndReplace(reBackSlash,	"\\\\",	b, new)
	searchAndReplace(reQuot,		"\\\"",	b, new)
	return stringOf b
}
