// ======================================================================
//
// ======================================================================


void prepareScopeRcrData(bool includeSeedRcr, string seedRcrID, rcrConditionString, Skip validAppliedRCs,
		Skip memRCsbyModule, Skip allRCsbyModule, allRCsNewAttrsbyModule, allRCsActionsbyModule,
		OleAutoObj obj_csApiLib, obj_csUser, com,
		bool includeAppliedMRCs,
		Skip scopeRCRs)
{
	csintDebug_("prepareScopeRcrData():")
	string errMsg
	Skip memRCsInOneModule = null
	for memRCsInOneModule in memRCsbyModule do
	{
		// ----------------------------------------------------
		// first, query for all approved rcrs in this module
		// ----------------------------------------------------
		string moduleID = (string key memRCsbyModule)
		string dbID = getDoorsDB_ID()
		string c = "(cvtype='problem')"
		string d = "has_dynamic_attrs_new(" ECPS_DOORSID " = 'req" moduleID "' and " ECPS_DOORS_DB_ID "='" dbID "')"
//		string rcrConditionString = buildApprovedStateQuery(m)	// we always use the configuration from the module we started from.
		string queryString = c " and " d " and " rcrConditionString
		csintDebug_("Query for all scoped " RCP "s involve this module is: " queryString)

		OleAutoObj rcrQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "", queryString, CSINT_RCP_DIALOG_DEFAULT_CR_ATTRS)
		if !null rcrQueryData then
		{
			int numApprovedRCRs = getNumberofObjectVectors(rcrQueryData, errMsg)
			csintDebug_ "Num of scoped " RCP "s found = " numApprovedRCRs ""
			int i
			for (i=0; i<numApprovedRCRs; i++)
			{
				OleAutoObj csObjectVector = getObjectVector(rcrQueryData, i, errMsg)
				string thisRcrNumber = getRecordField(csObjectVector, "problem_number",  errMsg)
				
				if !null validAppliedRCs then
				{
					if !find(validAppliedRCs, thisRcrNumber) then continue
				}

				string rcrStatus    = getRecordField(csObjectVector, "crstatus",  errMsg)
				string rcrName      = getRecordField(csObjectVector, "name",      errMsg)
				string rcrSubsystem = getRecordField(csObjectVector, "subsystem", errMsg)
				string rcr4pName = rcrSubsystem "/problem/" rcrName "/1"

				if !includeSeedRcr then
				{
					if thisRcrNumber == seedRcrID then continue
				}

				// =======================================
				// skip the RCR we already processed...
				// =======================================
				if find(scopeRCRs, thisRcrNumber) then
				{
					csintDebug_("Skip this RCR " thisRcrNumber)
					continue
				}
				put(scopeRCRs, thisRcrNumber, thisRcrNumber)

				// we need to query for all not-applied or applied MRCs, depending on value of includeAppliedMRCs
				string a = ""
				if (includeAppliedMRCs) then
					a = "(doorsIsApplied=TRUE)"
				else
					a = "(doorsIsApplied=FALSE)"
				string b = "is_dynamic_attrs_new_of('" rcr4pName "')"
				queryString = a " and " b
				csintDebug_("Query for all not-applied MRCs is: " queryString)
				OleAutoObj rcQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "Data_Report_dynamic_attrs_new", queryString, CSINT_RC_NEW_ATTRS)
				if !null rcQueryData then
				{
					int numRCs = getNumberofObjectVectors(rcQueryData, errMsg)
					csintDebug_ "for allRCs... Num of not-applied MRCs found = " numRCs ""
					int j
					for (j=0; j<numRCs; j++)
					{
						// a module RC...
						//
						csObjectVector = getObjectVector(rcQueryData, j, errMsg)
						Skip mrcAttrs = objectVectorToAttrSkip(csObjectVector, thisRcrNumber)
						string doorsID = getAttrValueFromAttrSkip(mrcAttrs, CSINT_RCM_RC_DOORSID)
						string moduleID = null
						string objectID = null
						ecpsSplitID(doorsID, moduleID, objectID)
						csintDebug_ "doorsID=" doorsID ", moduleID=" moduleID
						if !null moduleID then
						{
							// ====================================
							// Only process the MRC on the module also has mem MRC...
							// ====================================
							if !find(memRCsbyModule, moduleID) then
							{
								csintDebug_("Skip this mrc because it is not in the same module as any associated mem MRC.")
								delete mrcAttrs
								continue
							}

							Skip allRCsInOneModule = null
							if !find(allRCsbyModule, moduleID, allRCsInOneModule) then
							{
								allRCsInOneModule = createString
								put(allRCsbyModule, moduleID, allRCsInOneModule)
							}

							// -------------------------------------------
							// parsing the orc Attrs
							// -------------------------------------------
							Buffer mActionBuffer = create
							mActionBuffer += getAttrValueFromAttrSkip(mrcAttrs, "doorsChangeActions")

							int mPosition = 0
							string orcText = ""
							while (getValueBetweenTags(mPosition, mActionBuffer, H_X_ORC, orcText))
							{
								Buffer orcBuffer = create
								orcBuffer += orcText
								convertActionOrcToData(allRCsInOneModule, null, orcBuffer)
								delete orcBuffer
							}

							delete mActionBuffer

							// -------------------------------------------
							// get dynamic new data
							// -------------------------------------------
							string cvID = getAttrValueFromAttrSkip(mrcAttrs, "cvid")
							string new_attrs_data = null
							errMsg = getDynamicNewAttrWithCom(com, cvID, new_attrs_data)

							Skip allRCsNewAttrsInOneModule = null
							if !find(allRCsNewAttrsbyModule, moduleID,allRCsNewAttrsInOneModule) then
							{
								allRCsNewAttrsInOneModule = createString
								put(allRCsNewAttrsbyModule, moduleID, allRCsNewAttrsInOneModule)
							}

							Buffer newAttrsBuffer = create
							newAttrsBuffer += new_attrs_data
							mPosition = 0
							orcText = ""
							while (getValueBetweenTags(mPosition, newAttrsBuffer, H_X_ORC, orcText))
							{
								Buffer orcBuffer = create
								orcBuffer += orcText

								int sectionPos = 0
								string rcID = ""
								if (getValueBetweenTags(sectionPos, orcBuffer, "rcID", rcID)) then  // get rcID
								{
									csintDebug_("get new attrs for rcID=" rcID)
									Skip attrSkip = createString
									parseXMLforNewValues(com, orcBuffer, attrSkip, true)
									put(allRCsNewAttrsInOneModule, rcID, attrSkip)
								}
										
								delete orcBuffer
							}

							delete newAttrsBuffer
						}

						delete mrcAttrs
					}
					cleanupQueryData(rcQueryData)
				}
			}
			cleanupQueryData(rcrQueryData)
		}
	}

	Skip allRCsInOneModule = null
	for allRCsInOneModule in allRCsbyModule do
	{
		string moduleID = (string key allRCsbyModule)

		// for actions
		Skip allRCsActionsInOneModule = createString // rcID -> actions
		prepareRcActions(allRCsInOneModule, allRCsActionsInOneModule)
		put(allRCsActionsbyModule, moduleID, allRCsActionsInOneModule)
	}
}

// ======================================================================
//
// ======================================================================
void cleanupScopeRcrData(Skip &allRCsbyModule, &allRCsNewAttrsbyModule, &allRCsActionsbyModule)
{
	allRCsbyModule = cleanThreeLevelSkip(allRCsbyModule)
	csintDebug_("After clean allRCsbyModule")

	allRCsNewAttrsbyModule = cleanThreeLevelSkip(allRCsNewAttrsbyModule)
	csintDebug_("After clean allRCsNewAttrsbyModule")

	if !null allRCsActionsbyModule then
	{
		Skip allRCsActionsInOneModule
		for allRCsActionsInOneModule in allRCsActionsbyModule do
		{
			Skip objActions
			for objActions in allRCsActionsInOneModule do
			{
				releaseObjActions(objActions)
			}
			delete allRCsActionsInOneModule
		}
		delete allRCsActionsbyModule
	}
	csintDebug_("After clean allRCsActionsbyModule")
}

// ======================================================================
//
// ======================================================================
int prepareRcrData(string onlyThisModule, rcrID, fourPartName, OleAutoObj obj_csApiLib, obj_csUser, com, 
			Skip mrcCvids, mrcOrigCvids, memRCsbyModule, memRCsOrderByModule, memAnalystDataByModule, memRCsNewAttrsbyModule, memRCsActionsbyModule)
{
	csintDebug_("prepareRcrData(" rcrID "):")
	// ===========================================================
	// get member mrc (dynamic_attrs_new) objects of a selected RCR
	// ===========================================================
	int numMRCs = 0
	string queryString = "is_dynamic_attrs_new_of('" fourPartName "')"
	if !null onlyThisModule then queryString = "is_dynamic_attrs_new_of('" fourPartName "') and doorsID ='req" onlyThisModule "'"
	csintDebug_("query for associated mrcs...")
	csintDebug_("Query CS using Data_Report_dynamic_attrs_new report with query=" queryString)
	OleAutoObj rcQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "Data_Report_dynamic_attrs_new", queryString, CSINT_RC_NEW_ATTRS)
	csintDebug_("Query CS return.")
	string errMsg = null
	if !null rcQueryData then numMRCs = getNumberofObjectVectors(rcQueryData, errMsg)
	csintDebug_("Num of mrcs found = " numMRCs "")

	// =========================================================================
	// group member RCs by module because we need to detect conflicts per module
	// =========================================================================
	if (numMRCs > 0) then
	{
		int i
		for (i=0; i<numMRCs; i++)
		{
			csintDebug_("Looping for mrc[" i "]...")
			// a module RC...
			//
			OleAutoObj csObjectVector = getObjectVector(rcQueryData, i, errMsg)
			csintDebug_("Before convert cs data to skip")
			Skip mrcAttrs = objectVectorToAttrSkip(csObjectVector, rcrID)
			csintDebug_("After convert cs data to skip")
			string doorsID = getAttrValueFromAttrSkip(mrcAttrs, CSINT_RCM_RC_DOORSID)
			string moduleID = null
			string objectID = null
			ecpsSplitID(doorsID, moduleID, objectID)

			if !null moduleID then
			{
				// -------------------------------------------
				// create the data structure for this module
				// -------------------------------------------
				Skip memRCsInOneModule = null	// rcID -> orc attrs
				if !find(memRCsbyModule, moduleID, memRCsInOneModule) then
				{
					memRCsInOneModule = createString
					put(memRCsbyModule, moduleID, memRCsInOneModule)
				}

				Skip memRCsOrder = null	// i -> rcID
				if !find(memRCsOrderByModule, moduleID, memRCsOrder) then
				{
					memRCsOrder = create
					put(memRCsOrderByModule, moduleID, memRCsOrder)
				}

				Skip mAnalystData = null
				if !find(memAnalystDataByModule, moduleID, mAnalystData) then
				{
					mAnalystData = create
					put(memAnalystDataByModule, moduleID, mAnalystData)
				}

				Buffer mActionBuffer = create
				mActionBuffer = ""
				mActionBuffer += getAttrValueFromAttrSkip(mrcAttrs, "doorsChangeActions")
				string workingTempFile = tempFileName()
				csintDebug_("Using temp file " workingTempFile)
				Stream tempWorkingStream = write(workingTempFile, CP_UTF8)
				tempWorkingStream << mActionBuffer
				close tempWorkingStream
				setempty mActionBuffer
				readDoorsFile(workingTempFile, mActionBuffer)				
				deleteFile workingTempFile

				int mPosition = 0

				// -------------------------------------------
				// get analyst data
				csintDebug_("get analyst data")
				// -------------------------------------------
				string analystData = ""
				getValueBetweenTags(mPosition, mActionBuffer, H_X_ANALYST_DATA, analystData)
				string mExistOrigUmlFile = ""
				string mExistUmlFile = ""
				string mExistOrigStorageVersion = ""
				string mExistStorageVersion = ""
				string mExistOrigUmlGuid = ""
				string mExistUmlGuid = ""
				getAnalystDataFromMrcCache(analystData, mExistOrigUmlFile, mExistUmlFile, mExistOrigStorageVersion, mExistStorageVersion, mExistOrigUmlGuid, mExistUmlGuid)
				put(mAnalystData, 1, mExistOrigUmlFile)
				put(mAnalystData, 2, mExistUmlFile)
				put(mAnalystData, 3, mExistOrigStorageVersion)
				put(mAnalystData, 4, mExistStorageVersion)
				put(mAnalystData, 5, mExistOrigUmlGuid)
				put(mAnalystData, 6, mExistUmlGuid)

				// -------------------------------------------
				// parsing the orc Attrs
				csintDebug_("parsing the orc attrs")
				// -------------------------------------------
				string orcText = ""
				int rcOrder = 1
				while (getValueBetweenTags(mPosition, mActionBuffer, H_X_ORC, orcText))
				{
					Buffer orcBuffer = create
					orcBuffer += orcText
					string rcID = convertActionOrcToData(memRCsInOneModule, null, orcBuffer)
					delete orcBuffer

					put(memRCsOrder, rcOrder, rcID)
					rcOrder++
				}

				delete mActionBuffer

				// -------------------------------------------
				// remember cvids of new and orig
				csintDebug_("remember cvids")
				// -------------------------------------------
				string cvID = getAttrValueFromAttrSkip(mrcAttrs, "cvid")
				put(mrcCvids, moduleID, cvID)	// save it for later use
				csintDebug_("new cvid = " cvID)

				string origCvid = getOrigCvidFromRcAttrs(obj_csApiLib, obj_csUser, mrcAttrs)
				put(mrcOrigCvids, moduleID, origCvid)	// save it for later use
				csintDebug_("orig cvid = " origCvid)

				// -------------------------------------------
				// get dynamic new data
				csintDebug_("get dynamic new data")
				// -------------------------------------------
				string new_attrs_data = null
				errMsg = getDynamicNewAttrWithCom(com, cvID, new_attrs_data)

				Skip memRCsNewAttrsInOneModule = null
				if !find(memRCsNewAttrsbyModule, moduleID, memRCsNewAttrsInOneModule) then
				{
					memRCsNewAttrsInOneModule = createString
					put(memRCsNewAttrsbyModule, moduleID, memRCsNewAttrsInOneModule)
				}

				Buffer newAttrsBuffer = create
				newAttrsBuffer += new_attrs_data
				mPosition = 0
				orcText = ""
				while (getValueBetweenTags(mPosition, newAttrsBuffer, H_X_ORC, orcText))
				{
					Buffer orcBuffer = create
					orcBuffer += orcText

					int sectionPos = 0
					string rcID = ""
					if (getValueBetweenTags(sectionPos, orcBuffer, "rcID", rcID)) then  // get rcID
					{
						csintDebug_("get new attrs for rcID=" rcID)
						Skip attrSkip = createString
						parseXMLforNewValues(com, orcBuffer, attrSkip, true)
						put(memRCsNewAttrsInOneModule, rcID, attrSkip)
					}
						
					delete orcBuffer
				}

				delete newAttrsBuffer
			}

			delete mrcAttrs
		}
	}
	if !null rcQueryData then cleanupQueryData(rcQueryData)

	// ====================================
	// prepare memRCs actions
	csintDebug_("prepare mem actions data")
	// ====================================
	if (numMRCs >0) then
	{
		Skip memRCsInOneModule = null
		for memRCsInOneModule in memRCsbyModule do
		{
			string moduleID = (string key memRCsbyModule)

			// for actions
			Skip memRCsActionsInOneModule = createString // rcID -> actions
			prepareRcActions(memRCsInOneModule, memRCsActionsInOneModule)
			put(memRCsActionsbyModule, moduleID, memRCsActionsInOneModule)
		}
	}
	//debugPrintMemRCsActionsbyModule(memRCsActionsbyModule)

	return numMRCs
}

void cleanRcActionsByModule(Skip &memRCsActionsbyModule)
{
	if !null memRCsActionsbyModule then
	{
		Skip memRCsActionsInOneModule
		for memRCsActionsInOneModule in memRCsActionsbyModule do
		{
			//string moduleID = (string key memRCsActionsbyModule)
			//csintDebug_("    clean data for " moduleID)
			Skip objActions
			for objActions in memRCsActionsInOneModule do
			{
				//string cvId = (string key memRCsActionsInOneModule)
				//csintDebug_("      for " cvId)
				releaseObjActions(objActions)
			}
			delete memRCsActionsInOneModule
		}
		delete memRCsActionsbyModule
		memRCsActionsbyModule = null
	}
}

// ======================================================================
//
// ======================================================================
void cleanupRcrData(Skip &mrcCvids, &mrcOrigCvids, &memRCsbyModule, &memRCsOrderByModule, &memAnalystDataByModule, &memRCsNewAttrsbyModule, &memRCsActionsbyModule)
{
	if !null memRCsbyModule then memRCsbyModule = cleanThreeLevelSkip(memRCsbyModule)
	csintDebug_("After clean memRCsbyModule")

	if !null memRCsNewAttrsbyModule then memRCsNewAttrsbyModule = cleanThreeLevelSkip(memRCsNewAttrsbyModule)
	csintDebug_("After clean memRCsNewAttrsbyModule")

	if !null memRCsActionsbyModule then cleanRcActionsByModule(memRCsActionsbyModule)
	csintDebug_("After clean memRCsActionsbyModule")

	memRCsOrderByModule = cleanTwoLevelSkip(memRCsOrderByModule)
	csintDebug_("After clean memRCsOrderByModule")

	memAnalystDataByModule = cleanTwoLevelSkip(memAnalystDataByModule)
	csintDebug_("After clean memAnalystDataByModule")

	delete mrcCvids
	delete mrcOrigCvids
}
